@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,7 @@ package org.mozc.android.inputmethod.japanese;
 
 import org.mozc.android.inputmethod.japanese.mushroom.MushroomUtil;
 import org.mozc.android.inputmethod.japanese.ui.MenuDialog.MenuDialogListener;
+import com.google.common.base.Preconditions;
 
 import android.content.Context;
 import android.content.Intent;
@@ -46,7 +47,7 @@ class MozcMenuDialogListenerImpl implements MenuDialogListener {
   private boolean showInputMethodPicker = false;
 
   MozcMenuDialogListenerImpl(InputMethodService inputMethodService) {
-    this.inputMethodService = inputMethodService;
+    this.inputMethodService = Preconditions.checkNotNull(inputMethodService);
   }
 
   @Override
