@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -30,15 +30,18 @@
 package org.mozc.android.inputmethod.japanese;
 
 import org.mozc.android.inputmethod.japanese.FeedbackManager.FeedbackEvent;
-import org.mozc.android.inputmethod.japanese.HardwareKeyboard.CompositionSwitchMode;
-import org.mozc.android.inputmethod.japanese.JapaneseKeyboard.KeyboardSpecification;
 import org.mozc.android.inputmethod.japanese.KeycodeConverter.KeyEventInterface;
-import org.mozc.android.inputmethod.japanese.SymbolInputView.MajorCategory;
+import org.mozc.android.inputmethod.japanese.hardwarekeyboard.HardwareKeyboard.CompositionSwitchMode;
+import org.mozc.android.inputmethod.japanese.keyboard.Keyboard.KeyboardSpecification;
+import org.mozc.android.inputmethod.japanese.model.SymbolMajorCategory;
 import org.mozc.android.inputmethod.japanese.protobuf.ProtoCommands;
 import org.mozc.android.inputmethod.japanese.protobuf.ProtoCommands.Input.TouchEvent;
+import com.google.common.base.Optional;
 
 import java.util.List;
 
+import javax.annotation.Nullable;
+
 /**
  * Callback object for view evnets.
  *
@@ -53,29 +56,37 @@ public interface ViewEventListener {
    * @param touchEventList {@code TouchEvent} instances related to this key event for logging
    *        usage stats.
    */
-  public void onKeyEvent(ProtoCommands.KeyEvent mozcKeyEvent, KeyEventInterface keyEvent,
-                         KeyboardSpecification keyboardSpecification,
-                         List<? extends TouchEvent> touchEventList);
+  public void onKeyEvent(@Nullable ProtoCommands.KeyEvent mozcKeyEvent,
+                         @Nullable KeyEventInterface keyEvent,
+                         @Nullable KeyboardSpecification keyboardSpecification,
+                         List<TouchEvent> touchEventList);
 
   /**
    * Called when Undo is fired (by soft keyboard).
    * @param touchEventList {@code TouchEvent} instances related to this undo for logging
    *        usage stats.
    */
-  public void onUndo(List<? extends TouchEvent> touchEventList);
+  public void onUndo(List<TouchEvent> touchEventList);
 
   /**
    * Called when a conversion candidate is selected.
    *
    * @param candidateId the id which Candidate and CandidateWord has.
+   * @param rowIndex index of row in which the candidate is. If absent no stats are sent.
    */
-  public void onConversionCandidateSelected(int candidateId);
+  public void onConversionCandidateSelected(int candidateId, Optional<Integer> rowIndex);
+
+  /** Called when page down button is tapped. */
+  public void onPageUp();
+
+  /** Called when page down button is tapped. */
+  public void onPageDown();
 
   /**
    * Called when a candidate on symbol input view is selected.
-   * TODO(hidehiko): MajorCategory is in SymbolInputView. It's a bit weird structure. Refactor it.
    */
-  public void onSymbolCandidateSelected(MajorCategory majorCategory, String candidate);
+  public void onSymbolCandidateSelected(SymbolMajorCategory majorCategory, String candidate,
+                                        boolean updateHistory);
 
   /**
    * Called when a feedback event happens.
@@ -99,7 +110,8 @@ public interface ViewEventListener {
    * @param touchEventList {@code TouchEvent} instances which is related to this event
    *        for logging usage stats.
    */
-  public void onShowMenuDialog(List<? extends TouchEvent> touchEventList);
+  // TODO(matsuzakit): Rename. onFlushTouchEventStats ?
+  public void onShowMenuDialog(List<TouchEvent> touchEventList);
 
   /**
    * Called when the symbol input view is shown.
@@ -107,7 +119,7 @@ public interface ViewEventListener {
    * @param touchEventList {@code TouchEvent} instances which is related to this event
    *        for logging usage stats.
    */
-  public void onShowSymbolInputView(List<? extends TouchEvent> touchEventList);
+  public void onShowSymbolInputView(List<TouchEvent> touchEventList);
 
   /**
    * Called when the symbol input view is closed.
@@ -116,7 +128,7 @@ public interface ViewEventListener {
 
   /**
    * Called when the hardware_composition_button is clicked.
-   * @param newParam TODO(matsuzakit):
+   * @param mode new mode
    */
   public void onHardwareKeyboardCompositionModeChange(CompositionSwitchMode mode);
 
@@ -124,4 +136,15 @@ public interface ViewEventListener {
    * Called when the key for editor action is pressed.
    */
   public void onActionKey();
+
+  /** Called when the narrow mode of the view is changed. */
+  public void onNarrowModeChanged(boolean newNarrowMode);
+
+  /**
+   * Called when the keyboard layout preference should be updated.
+   * <p>
+   * The visible keyboard will also be updated as the result through a callback object.
+   */
+  public void onUpdateKeyboardLayoutAdjustment(
+      ViewManagerInterface.LayoutAdjustment layoutAdjustment);
 }
