@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -109,9 +109,8 @@ public class MozcMenuDialogListenerImplTest extends InstrumentationTestCaseWithM
   public void testLaunchPreferenceActivitySelected() {
     final ViewManagerInterface stubViewManager = createNiceMock(ViewManagerInterface.class);
     final Activity stubPreferenceActivity = createNiceMock(Activity.class);
-    final Activity stubSoftwareKeyboardAdvancedSettingActivity = createNiceMock(Activity.class);
 
-    Context context = createNiceMock(Context.class);
+    Context context = createNiceMock(MockContext.class);
     String packageName = "test.package.name";
 
     try {
@@ -130,13 +129,6 @@ public class MozcMenuDialogListenerImplTest extends InstrumentationTestCaseWithM
               return stubPreferenceActivity.getClass();
             }
 
-            @Override
-            public Optional<Class<? extends Activity>>
-                getSoftwareKeyboardAdvancedSettingActivityClass() {
-              return Optional.<Class<? extends Activity>>of(
-                  stubSoftwareKeyboardAdvancedSettingActivity.getClass());
-            }
-
             @Override
             public boolean isWelcomeActivityPreferrable() {
               return true;
