@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,6 +29,8 @@
 
 package org.mozc.android.inputmethod.japanese.session;
 
+import com.google.common.base.Optional;
+
 import android.app.Activity;
 import android.content.SharedPreferences;
 import android.test.InstrumentationTestCase;
@@ -36,7 +38,6 @@ import android.test.suitebuilder.annotation.SmallTest;
 
 import java.io.IOException;
 import java.net.ServerSocket;
-import java.net.Socket;
 
 /**
  */
@@ -54,7 +55,7 @@ public class SessionHandlerFactoryTest extends InstrumentationTestCase {
     @Override
     public void run() {
       try {
-        Socket socket = serverSocket.accept();
+        serverSocket.accept();
       } catch (IOException e) {
         // Do nothing.
       }
@@ -81,14 +82,6 @@ public class SessionHandlerFactoryTest extends InstrumentationTestCase {
     super.tearDown();
   }
 
-  @SmallTest
-  public void testCreateSessionHandler_nullParameter() {
-    // SessionHandler instance for null parameter.
-    assertEquals(
-        LocalSessionHandler.class,
-        new SessionHandlerFactory(SharedPreferences.class.cast(null)).create().getClass());
-  }
-
   @SmallTest
   public void testCreateSessionHandler_appropriateParameter() {
       // SessionHandlerSocketClient for appropriate preference.
@@ -100,7 +93,7 @@ public class SessionHandlerFactoryTest extends InstrumentationTestCase {
                      Integer.toString(PORT));
     editor.commit();
     assertEquals(SocketSessionHandler.class,
-                 new SessionHandlerFactory(sharedPreferences).create().getClass());
+                 new SessionHandlerFactory(Optional.of(sharedPreferences)).create().getClass());
   }
 
   @SmallTest
@@ -114,7 +107,7 @@ public class SessionHandlerFactoryTest extends InstrumentationTestCase {
                      Integer.toString(PORT));
     editor.commit();
     assertEquals(LocalSessionHandler.class,
-                 new SessionHandlerFactory(sharedPreferences).create().getClass());
+                 new SessionHandlerFactory(Optional.of(sharedPreferences)).create().getClass());
   }
 
   @SmallTest
@@ -128,6 +121,6 @@ public class SessionHandlerFactoryTest extends InstrumentationTestCase {
                      "MALFORMED");
     editor.commit();
     assertEquals(LocalSessionHandler.class,
-                 new SessionHandlerFactory(sharedPreferences).create().getClass());
+                 new SessionHandlerFactory(Optional.of(sharedPreferences)).create().getClass());
   }
 }
