@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -34,6 +34,10 @@
 #include <aclapi.h>
 #endif  // OS_WIN
 
+#ifdef OS_MACOSX
+#include <unistd.h>
+#endif  // OS_MACOSX
+
 #ifdef OS_LINUX
 #include <unistd.h>
 #include <sys/types.h>
@@ -50,10 +54,10 @@
 namespace mozc {
 namespace {
 
+#ifdef OS_WIN
 const wchar_t kElevatedProcessDisabledName[]
 = L"elevated_process_disabled";
 
-#ifdef OS_WIN
 // Returns true if both array have the same content.
 template <typename T, size_t ArraySize>
 bool AreEqualArray(const T (&lhs)[ArraySize], const T (&rhs)[ArraySize]) {
