@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -83,6 +83,9 @@ class SystemUtil {
   // <server directory>/documents but it can change among platforms.
   static string GetDocumentDirectory();
 
+  // Returns the directory path crash dumps are stored.
+  static string GetCrashReportDirectory();
+
   // return the username.  This function's name was GetUserName.
   // Since Windows reserves GetUserName as a macro, we have changed
   // the name to GetUserNameAsString.
