@@ -1,6 +1,6 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
-# Copyright 2010-2014, Google Inc.
+# Copyright 2010-2015, Google Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -34,7 +34,6 @@
 Typical usage:
 
   % python build_mozc.py gyp
-  % python build_mozc.py build_tools -c Release
   % python build_mozc.py build base/base.gyp:base
 """
 
@@ -47,9 +46,7 @@ import os
 import re
 import sys
 
-from build_tools import android_util
 from build_tools import mozc_version
-from build_tools.android_util import Emulator
 from build_tools.mozc_version import GenerateVersionFile
 from build_tools.test_tools import test_launcher
 from build_tools.util import CheckFileOrDie
@@ -68,6 +65,11 @@ from build_tools.util import RemoveFile
 from build_tools.util import RunOrDie
 from build_tools.util import RunOrDieError
 
+if not IsWindows():
+  # android_util depends on fcntl module which doesn't exist in Windows.
+  # pylint: disable=g-import-not-at-top
+  from build_tools.android_util import Emulator
+
 SRC_DIR = '.'
 # We need to obtain the absolute path of this script before we
 # call MoveToTopLevelSourceDirectory(), which may change the
@@ -85,7 +87,7 @@ def GetMozcVersion():
   return mozc_version.MozcVersion('%s/mozc_version.txt' % SRC_DIR)
 
 
-def GetBuildBaseName(options, target_platform):
+def GetBuildShortBaseName(options, target_platform):
   """Returns the build base directory.
 
   Determination priority is;
@@ -112,10 +114,11 @@ def GetBuildBaseName(options, target_platform):
   build_base = ''
 
   generator = GetGeneratorName(options.ensure_value('gyp_generator', None))
-  if generator == 'ninja':
-    build_base = 'out'
-  elif generator == 'msvs':
-    build_base = 'out_win'
+  if target_platform == 'Windows':
+    if generator == 'ninja':
+      build_base = 'out_win'
+    else:
+      build_base = 'out_win_msvs'
   elif target_platform == 'Mac':
     build_base = 'out_mac'
   elif target_platform == 'Linux':
@@ -127,6 +130,12 @@ def GetBuildBaseName(options, target_platform):
   else:
     raise RuntimeError('Unsupported platform: %s' % target_platform)
 
+  return build_base
+
+
+def GetBuildBaseName(options, target_platform):
+  build_base = GetBuildShortBaseName(options, target_platform)
+
   # On Linux, seems there is no way to specify the build_base directory
   # inside common.gypi
   if IsWindows() or IsMac():
@@ -139,12 +148,10 @@ def GetGeneratorName(generator):
   """Gets the generator name based on the platform."""
   if generator:
     return generator
-  elif IsWindows():
-    return 'ninja'
   elif IsMac():
     return 'xcode'
   else:
-    return 'make'
+    return 'ninja'
 
 
 def GetPkgConfigCommand():
@@ -193,7 +200,6 @@ def GetGypFileNames(options):
 
   for name in mozc_top_level_names:
     gyp_file_names.extend(glob.glob(name + '/*.gyp'))
-  gyp_file_names.extend(glob.glob('%s/build_tools/*/*.gyp' % SRC_DIR))
   # Include subdirectories of data/test/session/scenario
   gyp_file_names.extend(glob.glob('%s/data/test/session/scenario/*.gyp' %
                                   SRC_DIR))
@@ -217,6 +223,9 @@ def GetGypFileNames(options):
   elif options.target_platform == 'NaCl':
     # Add chrome NaCl Mozc gyp scripts.
     gyp_file_names.append('%s/chrome/nacl/nacl_extension.gyp' % SRC_DIR)
+  elif options.target_platform == 'Android':
+    # Add Android Mozc gyp scripts.
+    gyp_file_names.extend(glob.glob('%s/android/*/*.gyp' % SRC_DIR))
   gyp_file_names.sort()
   return gyp_file_names
 
@@ -229,8 +238,11 @@ def GetTopLevelSourceDirectoryName():
   num_components = len(SRC_DIR.split('/'))
   return os.path.join(script_file_directory_name, *(['..'] * num_components))
 
-
-
+def GetAdditionalThirdPartyDir():
+  """Returns the addtitional third party dir."""
+  additional_third_party_path = 'third_party'
+  return os.path.abspath(os.path.join(GetTopLevelSourceDirectoryName(),
+                                      additional_third_party_path))
 
 def GetAndroidHome(options):
   """Gets the root directory of the SDK from options, ANDROID_HOME or PATH."""
@@ -342,12 +354,6 @@ def ParseGypOptions(args=None, values=None):
   parser.add_option('--qtdir', dest='qtdir',
                     default=os.getenv('QTDIR', None),
                     help='Qt base directory to be used.')
-  parser.add_option('--channel_dev', action='store', dest='channel_dev',
-                    type='int',
-                    help='Pass --channel_dev=1 if you need to build mozc with '
-                    'the CHANNEL_DEV macro enabled. Pass 0 if you don\'t need '
-                    'the macro. If --channel_dev= flag is not passed, Mozc '
-                    'version is used to deretmine if the macro is necessary.')
   parser.add_option('--version_file', dest='version_file',
                     help='use the specified version template file',
                     default='mozc_version_template.txt')
@@ -375,10 +381,20 @@ def ParseGypOptions(args=None, values=None):
   # Android
   parser.add_option('--android_arch', dest='android_arch',
                     type='choice',
-                    choices=('arm', 'x86', 'mips'),
+                    choices=('arm', 'x86', 'mips', 'arm64', 'x86_64', 'mips64'),
                     default='arm',
                     help='[Android build only] Android architecture '
                     '(arm, x86, mips)')
+  parser.add_option('--android_stl', dest='android_stl',
+                    type='choice',
+                    choices=('gnustl', 'libcxx'),
+                    default='libcxx',
+                    help='[Android build only] Standard C++ library')
+  parser.add_option('--android_compiler', dest='android_compiler',
+                    type='choice',
+                    choices=('gcc', 'clang'),
+                    default='gcc',
+                    help='[Android build only] Compiler')
   parser.add_option('--android_application_id', dest='android_application_id',
                     default='org.mozc.android.inputmethod.japanese',
                     help='[Android build only] Android\'s application id'
@@ -469,10 +485,10 @@ def ParseGypOptions(args=None, values=None):
                       help='A path to the binary directory of wix.')
 
     # For internal Windows builds, gyp is expected to generate solution files
-    # for Visual Studio 2010, which is a default compiler for Mozc.  However,
+    # for Visual Studio 2013, which is a default compiler for Mozc.  However,
     # you can specify the target version explicitly via 'msvs_version' option
     # as follows.
-    parser.add_option('--msvs_version', dest='msvs_version', default='2010',
+    parser.add_option('--msvs_version', dest='msvs_version', default='2013',
                       help='Specifies the target MSVS version.')
 
   return parser.parse_args(args, values)
@@ -509,7 +525,7 @@ def ExpandMetaTarget(options, meta_target_name):
   elif target_platform == 'Mac':
     targets = [SRC_DIR + '/mac/mac.gyp:DiskImage']
   elif target_platform == 'Windows':
-    targets = ['out/%s:mozc_win32_build32' % options.configuration]
+    targets = ['out_win/%s:mozc_win32_build32' % options.configuration]
     build_dir = os.path.abspath(os.path.join(
         GetBuildBaseName(options, target_platform),
         '%sDynamic' % options.configuration))
@@ -518,9 +534,9 @@ def ExpandMetaTarget(options, meta_target_name):
     if os.path.exists(qtcore_dll) or os.path.exists(qtcored_dll):
       # This means that Mozc is configured to use DLL versioin of Qt.
       # Let's build Mozc with DLL version of C++ runtime for the compatibility.
-      targets += [
-          'out/%sDynamic:mozc_win32_build32_dynamic' % options.configuration]
-    targets.append('out/%s_x64:mozc_win32_build64' % options.configuration)
+      targets += ['out_win/%sDynamic:mozc_win32_build32_dynamic'
+                  % options.configuration]
+    targets.append('out_win/%s_x64:mozc_win32_build64' % options.configuration)
   elif target_platform == 'NaCl':
     targets = [SRC_DIR + '/chrome/nacl/nacl_extension.gyp:nacl_mozc']
 
@@ -540,6 +556,15 @@ def ParseBuildOptions(args=None, values=None):
                     default='Debug', help='specify the build configuration.')
 
   (options, args) = parser.parse_args(args, values)
+
+  # Debug_Android and Release_Android have been deprecated.
+  # As migration process, throw exception with description.
+  # TODO(matsuzakit): Remove this block later.
+  if (options.configuration == 'Debug_Android'
+      or options.configuration == 'Release_Android'):
+    raise RuntimeError('%s has been deprecated. Use Debug or Release instead.'
+                       % options.configuration)
+
   targets = []
   for arg in args:
     targets.extend(ExpandMetaTarget(options, arg))
@@ -571,10 +596,6 @@ def ParseRunTestsOptions(args=None, values=None):
                     help='[Android build only] specify which emulator/device '
                     'you test on. '
                     'If not specified emulators are launched and used.')
-  parser.add_option('--android_min_port', dest='android_min_port',
-                    default='5554',
-                    help='Minimum port number of emulators which will be '
-                    'launched by this script (inclusive).')
 
   return parser.parse_args(args, values)
 
@@ -617,12 +638,11 @@ def AddPythonPathToEnvironmentFilesForWindows(out_dir):
 def GypMain(options, unused_args, _):
   """The main function for the 'gyp' command."""
   # Generate a version definition file.
-  # The version file is used in this method to check if this is dev channel.
   logging.info('Generating version definition file...')
   template_path = '%s/%s' % (SRC_DIR, options.version_file)
   version_path = '%s/mozc_version.txt' % SRC_DIR
   GenerateVersionFile(template_path, version_path, options.target_platform,
-                      options.channel_dev, options.android_application_id,
+                      options.android_application_id,
                       options.android_arch)
   version = GetMozcVersion()
   target_platform = version.GetTargetPlatform()
@@ -690,6 +710,8 @@ def GypMain(options, unused_args, _):
 
   mozc_root = os.path.abspath(GetTopLevelSourceDirectoryName())
   gyp_options.extend(['-D', 'abs_depth=%s' % mozc_root])
+  gyp_options.extend(['-D', ('additional_third_party_dir=%s'
+                             % GetAdditionalThirdPartyDir())])
 
   gyp_options.extend(['-D', 'python_executable=%s' % sys.executable])
 
@@ -736,11 +758,15 @@ def GypMain(options, unused_args, _):
 
   gyp_options.extend(['-D', 'android_home=%s' % android_home])
   gyp_options.extend(['-D', 'android_arch=%s' % options.android_arch])
+  gyp_options.extend(['-D', 'android_stl=%s' % options.android_stl])
+  gyp_options.extend(['-D', 'android_compiler=%s' % options.android_compiler])
   gyp_options.extend(['-D', 'android_ndk_home=%s' % android_ndk_home])
   gyp_options.extend(['-D', 'android_application_id=%s' %
-                       options.android_application_id])
+                      options.android_application_id])
   gyp_options.extend(['-D', 'build_base=%s' %
-                       GetBuildBaseName(options, target_platform)])
+                      GetBuildBaseName(options, target_platform)])
+  gyp_options.extend(['-D', 'build_short_base=%s' %
+                      GetBuildShortBaseName(options, target_platform)])
 
   disable_unittest_if_not_available = True
   if disable_unittest_if_not_available:
@@ -768,12 +794,7 @@ def GypMain(options, unused_args, _):
 
   gyp_options.extend(['-D', 'mac_dir=%s' % mac_dir])
 
-  # Check the version and determine if the building version is dev-channel or
-  # not. Note that if --channel_dev is explicitly set, we don't check the
-  # version number.
-  if options.channel_dev is None:
-    options.channel_dev = version.IsDevChannel()
-  if options.channel_dev:
+  if version.IsDevChannel():
     gyp_options.extend(['-D', 'channel_dev=1'])
 
   def SetCommandLineForFeature(option_name, windows=False, mac=False,
@@ -818,7 +839,7 @@ def GypMain(options, unused_args, _):
                         '%s=%s' % (enable_option_name, 1 if enabled else 0)])
 
   is_official = (options.branding == 'GoogleJapaneseInput')
-  is_official_dev = (is_official and options.channel_dev)
+  is_official_dev = (is_official and version.IsDevChannel())
 
   SetCommandLineForFeature(option_name='cloud_handwriting',
                            linux=is_official_dev,
@@ -858,7 +879,7 @@ def GypMain(options, unused_args, _):
   else:
     gyp_options.extend(['-D', 'use_dynamically_linked_qt=0'])
 
-  if options.use_zinnia:
+  if options.use_zinnia and target_platform not in ['Android', 'NaCl']:
     gyp_options.extend(['-D', 'use_zinnia=YES'])
   else:
     gyp_options.extend(['-D', 'use_zinnia=NO'])
@@ -902,18 +923,31 @@ def GypMain(options, unused_args, _):
     gyp_options.extend(['-D', 'pkg_config_command='])
 
   if target_platform == 'NaCl':
-    if os.path.isdir(options.nacl_sdk_root):
+    if options.nacl_sdk_root:
       nacl_sdk_root = os.path.abspath(options.nacl_sdk_root)
     else:
-      PrintErrorAndExit('The directory specified with --nacl_sdk_root does not '
-                        'exist: %s' % options.nacl_sdk_root)
+      nacl_sdk_root = os.path.abspath(os.path.join(GetAdditionalThirdPartyDir(),
+                                                   'nacl_sdk', 'pepper_27'))
+    if not os.path.isdir(nacl_sdk_root):
+      PrintErrorAndExit('The nacl_sdk_root directory (%s) does not exist.'
+                        % options.nacl_sdk_root)
     gyp_options.extend(['-D', 'nacl_sdk_root=%s' % nacl_sdk_root])
-    gyp_options.extend(['-D', 'clang=1'])
 
   if options.server_dir:
     gyp_options.extend([
         '-D', 'server_dir=%s' % os.path.abspath(options.server_dir)])
 
+  # TODO(yukawa): Use ninja on OSX.
+  if generator == 'ninja':
+    gyp_options.extend(['--generator-output=.'])
+    short_basename = GetBuildShortBaseName(options, target_platform)
+    gyp_options.extend(['-G', 'output_dir=%s' % short_basename])
+
+  # Enable cross-compile
+  # TODO(yukawa): Enable GYP_CROSSCOMPILE for Windows.
+  if not IsWindows():
+    os.environ['GYP_CROSSCOMPILE'] = '1'
+
   # Run GYP.
   logging.info('Running GYP...')
   RunOrDie(gyp_command + gyp_options)
@@ -927,7 +961,7 @@ def GypMain(options, unused_args, _):
 
   # For internal Ninja build on Windows, set up environment files
   if IsWindows() and generator == 'ninja':
-    out_dir = os.path.join(GetTopLevelSourceDirectoryName(), 'out')
+    out_dir = os.path.join(GetTopLevelSourceDirectoryName(), 'out_win')
     AddPythonPathToEnvironmentFilesForWindows(out_dir)
 
   # When Windows build is configured to use DLL version of Qt, copy Qt's DLLs
@@ -948,7 +982,6 @@ def GypMain(options, unused_args, _):
         GetBuildScriptDirectoryName(), 'build_tools', 'copy_dll_and_symbol.py')
     copy_modes = [
         {'configuration': 'DebugDynamic', 'basenames': 'QtCored4;QtGuid4'},
-        {'configuration': 'OptimizeDynamic', 'basenames': 'QtCore4;QtGui4'},
         {'configuration': 'ReleaseDynamic', 'basenames': 'QtCore4;QtGui4'}]
     for mode in copy_modes:
       copy_commands = [
@@ -963,31 +996,6 @@ def GypMain(options, unused_args, _):
 
 
 
-def BuildToolsMain(options, unused_args, original_directory_name):
-  """The main function for 'build_tools' command."""
-  build_tools_dir = os.path.join(GetRelPath(os.getcwd(),
-                                            original_directory_name),
-                                 SRC_DIR, 'build_tools')
-  # Build targets in this order.
-  if IsWindows():
-    build_tools_targets = [
-        'out/%s:primitive_tools' % options.configuration,
-        'out/%s:build_tools' % options.configuration,
-    ]
-  else:
-    gyp_files = [
-        os.path.join(build_tools_dir, 'primitive_tools', 'primitive_tools.gyp'),
-        os.path.join(build_tools_dir, 'build_tools.gyp')
-    ]
-    build_tools_targets = []
-    for gyp_file in gyp_files:
-      (target, _) = os.path.splitext(os.path.basename(gyp_file))
-      build_tools_targets += ['%s:%s' % (gyp_file, target)]
-
-  for build_tools_target in build_tools_targets:
-    BuildMain(options, [build_tools_target], original_directory_name)
-
-
 def CanonicalTargetToGypFileAndTargetName(target):
   """Parses the target string."""
   if ':' not in target:
@@ -1004,33 +1012,17 @@ def BuildOnLinux(options, targets, unused_original_directory_name):
         CanonicalTargetToGypFileAndTargetName(target))
     target_names.append(target_name)
 
-  make_command = os.getenv('BUILD_COMMAND', 'make')
-  # flags for building in Chrome OS chroot environment
-  envvars = [
-      'CFLAGS',
-      'CXXFLAGS',
-      'CXX',
-      'CC',
-      'AR',
-      'AS',
-      'RANLIB',
-      'LD',
-  ]
-  for envvar in envvars:
-    if envvar in os.environ:
-      os.environ[envvar] = os.getenv(envvar)
+  ninja = 'ninja'
 
-  build_args = ['-j%s' % options.jobs,
-                'MAKE_JOBS=%s' % options.jobs,
-                'BUILDTYPE=%s' % options.configuration]
   if hasattr(options, 'android_device'):
     # Only for android testing.
-    build_args.append('ANDROID_DEVICES=%s' % options.android_device)
-
-  build_args.append('builddir_name=%s' %
-                    GetBuildBaseName(options,
-                                     GetMozcVersion().GetTargetPlatform()))
+    os.environ['ANDROID_DEVICES'] = options.android_device
 
+  short_basename = GetBuildShortBaseName(options,
+                                         GetMozcVersion().GetTargetPlatform())
+  make_command = ninja
+  build_args = ['-j %s' % options.jobs,
+                '-C', '%s/%s' % (short_basename, options.configuration)]
   RunOrDie([make_command] + build_args + target_names)
 
 
@@ -1178,53 +1170,76 @@ def RunTestsOnAndroid(options, build_args, original_directory_name):
     if options.android_device:
       serialnumbers.append(options.android_device)
     else:
-      # Set up the environment.
-      # Copy ANDROID_SDK_HOME directory.
-      # Copied directory will be overwritten by StartAndroidEmulator
-      # (inflating compressed file and being added emulator's temp files).
-      # Thus this setup must be done only once.
-      android_sdk_home_original = os.path.join(
-          SRC_DIR, 'android', 'android_sdk_home')
+      # Temporary AVDs are created beneath android_sdk_home.
       android_sdk_home = os.path.join(
           GetBuildBaseName(options, GetMozcVersion().GetTargetPlatform()),
           options.configuration,
           'android_sdk_home')
       android_home = GetAndroidHome(options)
-      android_util.SetUpTestingSdkHomeDirectory(android_sdk_home_original,
-                                                android_sdk_home,
-                                                android_home)
-      available_ports = [i for i
-                         in android_util.GetAvailableEmulatorPorts(android_home)
-                         if i >= int(options.android_min_port)]
-      if GetMozcVersion().GetAndroidArch() == 'arm':
-        acceptable_abi = ['armeabi', 'armeabi-v7a']
+
+      android_arch = GetMozcVersion().GetAndroidArch()
+      if android_arch == 'arm':
+        avd_configs = [
+            {'--name': 'Nexus5-Api21-arm-WVGA800',
+             '--target': 'android-21',
+             '--abi': 'default/armeabi-v7a',
+             '--device': 'Nexus 5',
+             '--skin': 'WVGA800'},
+            {'--name': 'Nexus10-Api21-arm-WXGA800',
+             '--target': 'android-21',
+             '--abi': 'default/armeabi-v7a',
+             '--device': 'Nexus 10',
+             '--skin': 'WXGA800'},]
+      elif android_arch == 'x86':
+        avd_configs = [
+            {'--name': 'Nexus5-Api21-x86-WVGA800',
+             '--target': 'android-21',
+             '--abi': 'default/x86',
+             '--device': 'Nexus 5',
+             '--skin': 'WVGA800'},
+            {'--name': 'Nexus10-Api21-x86-WXGA800',
+             '--target': 'android-21',
+             '--abi': 'default/x86',
+             '--device': 'Nexus 10',
+             '--skin': 'WXGA800'},]
+      elif android_arch == 'mips':
+        avd_configs = [
+            {'--name': 'NexusS-Api15-mips-WVGA800',
+             '--target': 'android-15',
+             '--abi': 'default/mips',
+             '--device': 'Nexus S',
+             '--skin': 'WVGA800'},
+            # As of 2014-05-27, the latest target of MIPS is 17.
+            {'--name': 'Nexus5-Api17-mips-WVGA800',
+             '--target': 'android-17',
+             '--abi': 'default/mips',
+             '--device': 'Nexus 5',
+             '--skin': 'WVGA800'},
+            {'--name': 'Nexus10-Api17-mips-WXGA800',
+             '--target': 'android-17',
+             '--abi': 'default/mips',
+             '--device': 'Nexus 10',
+             '--skin': 'WXGA800'},]
       else:
-        acceptable_abi = [GetMozcVersion().GetAndroidArch()]
-      avd_names = [i for i in android_util.GetAvdNames(android_sdk_home)
-                   if android_util.GetAvdProperties(
-                       android_sdk_home, i)['abi.type'] in acceptable_abi]
-      logging.info('Launching following AVDs; %s', avd_names)
-      if len(available_ports) < len(avd_names):
-        PrintErrorAndExit('available_ports (%d) is smaller than avd_names (%d)'
-                          % (len(available_ports), len(avd_names)))
-      emulators = []
-      # Invokes all the available emulators.
-      for avd_name in avd_names:
-        emulator = Emulator(android_sdk_home, avd_name, android_home)
-        logging.info('Creating SD card for %s', avd_name)
-        emulator.CreateBlankSdCard(300 * 1024 * 1024)
-        logging.info('Launching %s at port %d', avd_name, available_ports[0])
-        emulator.Launch(available_ports[0])
-        logging.info('Waiting for %s', avd_name)
-        emulator.GetAndroidDevice(android_home).WaitForDevice()
-        available_ports = available_ports[1:]
-        emulators.append(emulator)
-        serialnumbers.append(emulator.serial)
-        logging.info('Successfully launched %s', avd_name)
+        avd_configs = []
+
+      emulators = Emulator.LaunchAll(android_sdk_home, avd_configs,
+                                     android_home)
+      serialnumbers.extend([emulator.serial for emulator in emulators])
 
     # Run native and Java tests.
+    # If --configuration is Release, Java tests are skipped.
+    if options.configuration == 'Release':
+      targets = ['run_native_test']
+      logging.info('As this is Relase configuration, Java tests are skipped.')
+    else:
+      # run_java_test must be executed after run_native_test
+      # because package manager, which is mandatory to run Java tests,
+      # requires minutes to get ready.
+      targets = ['run_native_test', 'run_java_test']
+
     android_gyp = os.path.join(SRC_DIR, 'android', 'android.gyp')
-    for target in ('run_native_test', 'run_java_test'):
+    for target in targets:
       (build_options, build_targets) = ParseBuildOptions(
           build_args + ['%s:%s' % (android_gyp, target)])
       # Injects android_device attribute to build_options.
@@ -1304,9 +1319,12 @@ def RunTestsMain(options, args, original_directory_name):
     # TODO(yukawa): Change the notation rule of 'targets' to reduce the gap
     # between Ninja and make.
     if target_platform == 'Windows':
-      targets.append('out/%s:unittests' % options.configuration)
+      targets.append('out_win/%s:unittests' % options.configuration)
     else:
       targets.append('%s/gyp/tests.gyp:unittests' % SRC_DIR)
+    if target_platform == 'Android' and options.configuration != 'Release':
+      targets.append('%s/android/android.gyp:build_java_test' % SRC_DIR)
+
 
   # Build the test targets
   (build_opts, build_args) = ParseBuildOptions(build_options + targets)
@@ -1335,45 +1353,31 @@ def CleanBuildFilesAndDirectories(options, unused_args):
                       '*.vcxproj.filters', '*.vcxproj.user', 'gen_*_files.xml']:
         file_names.extend(glob.glob(os.path.join(gyp_directory_name,
                                                  pattern)))
-      for build_type in ['Debug', 'Optimize', 'Release']:
+      for build_type in ['Debug', 'Release']:
         directory_names.append(os.path.join(gyp_directory_name,
                                             build_type))
     elif IsMac():
       directory_names.extend(glob.glob(os.path.join(gyp_directory_name,
                                                     '*.xcodeproj')))
-    elif IsLinux():
-      file_names.extend(glob.glob(os.path.join(gyp_directory_name,
-                                               '*.target.mk')))
-      file_names.extend(glob.glob(os.path.join(gyp_directory_name,
-                                               '*/*.target.mk')))
-      file_names.extend(glob.glob(os.path.join(gyp_directory_name,
-                                               '*.host.mk')))
-      file_names.extend(glob.glob(os.path.join(gyp_directory_name,
-                                               '*/*.host.mk')))
-      file_names.extend(glob.glob(os.path.join(gyp_directory_name,
-                                               '*.Makefile')))
-      file_names.extend(glob.glob(os.path.join(gyp_directory_name,
-                                               '*/*.Makefile')))
   file_names.append('%s/mozc_version.txt' % SRC_DIR)
-  # Collect stuff in the top-level directory.
-  directory_names.append('mozc_build_tools')
 
   target_platform = GetBuildBaseName(options,
                                      GetMozcVersion().GetTargetPlatform())
   if target_platform:
     directory_names.append(target_platform)
   if IsLinux():
-    file_names.append('Makefile')
     # Remove auto-generated files.
     file_names.append(os.path.join(SRC_DIR, 'android', 'AndroidManifest.xml'))
     file_names.append(os.path.join(
         SRC_DIR, 'android', 'tests', 'AndroidManifest.xml'))
+    # Remove a symbolic link to android/resources/res
+    file_names.append(os.path.join(SRC_DIR, 'android', 'resources', 'res'))
     directory_names.append(os.path.join(SRC_DIR, 'android', 'assets'))
     # Delete files/dirs generated by Android SDK/NDK.
     android_library_projects = [
         '',
         'protobuf',
-        'resources_oss',
+        'resources',
         'tests',
         ]
     android_generated_dirs = ['bin', 'gen', 'obj', 'libs', 'gen_for_adt']
@@ -1381,10 +1385,6 @@ def CleanBuildFilesAndDirectories(options, unused_args):
       for directory in android_generated_dirs:
         directory_names.append(
             os.path.join(SRC_DIR, 'android', project, directory))
-    # In addition, remove resources/res/raw directory, which contains
-    # generated .pic files.
-    directory_names.append(
-        os.path.join(SRC_DIR, 'android', 'resources', 'res', 'raw'))
 
   # Remove files.
   for file_name in file_names:
@@ -1405,7 +1405,6 @@ def ShowHelpAndExit():
   print 'Commands: '
   print '  gyp          Generate project files.'
   print '  build        Build the specified target.'
-  print '  build_tools  Build tools used by the build command.'
   print '  runtests     Build all tests and run them.'
   print '  clean        Clean all the build files and directories.'
   print ''
@@ -1431,7 +1430,6 @@ def main():
 
   command_to_procedure = {
       'gyp': (ParseGypOptions, GypMain),
-      'build_tools': (ParseBuildOptions, BuildToolsMain),
       'build': (ParseBuildOptions, BuildMain),
       'runtests': (ParseRunTestsOptions, RunTestsMain),
       'clean': (ParseCleanOptions, CleanMain)}
