@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -45,12 +45,10 @@
 #include "config/config.pb.h"
 
 namespace mozc {
-
 namespace config {
-
 namespace {
 
-static const char kFileNamePrefix[] = "user://config";
+const char kFileNamePrefix[] = "user://config";
 
 void AddCharacterFormRule(const char *group,
                           const Config::CharacterForm preedit_form,
@@ -63,6 +61,19 @@ void AddCharacterFormRule(const char *group,
   rule->set_conversion_character_form(conversion_form);
 }
 
+bool GetPlatformSpecificDefaultEmojiSetting() {
+  // Disable Unicode emoji conversion by default on specific platforms.
+  bool use_emoji_conversion_default = true;
+#if defined(OS_WIN)
+  if (!SystemUtil::IsWindows8OrLater()) {
+    use_emoji_conversion_default = false;
+  }
+#elif defined(OS_ANDROID)
+  use_emoji_conversion_default = false;
+#endif
+  return use_emoji_conversion_default;
+}
+
 class ConfigHandlerImpl {
  public:
   ConfigHandlerImpl() {
@@ -137,33 +148,14 @@ bool ConfigHandlerImpl::SetConfigInternal(const Config &config) {
 
   // Initialize platform specific configuration.
   if (stored_config_.session_keymap() == Config::NONE) {
-#ifdef OS_MACOSX
-    stored_config_.set_session_keymap(Config::KOTOERI);
-#else  // OS_MACOSX
-    stored_config_.set_session_keymap(Config::MSIME);
-#endif  // OS_MACOSX
+    stored_config_.set_session_keymap(ConfigHandler::GetDefaultKeyMap());
   }
 
-#ifdef OS_ANDROID
-#ifdef CHANNEL_DEV
-  stored_config_.mutable_general_config()
-      ->set_upload_usage_stats(true);
-#endif  // CHANNEL_DEV
-#endif  // OS_ANDROID
+#if defined(OS_ANDROID) && defined(CHANNEL_DEV)
+  stored_config_.mutable_general_config()->set_upload_usage_stats(true);
+#endif  // CHANNEL_DEV && OS_ANDROID
 
-  // Enable Unicode emoji conversion in default on specific platforms.
-  // NaCl Mozc or Windows 8 or Mac 10.7(Lion) or later.
-#ifdef __native_client__
-  bool use_emoji_conversion_default = true;
-#else  // __native_client__
-  bool use_emoji_conversion_default = false;
-  if (SystemUtil::MacOSVersionIsGreaterOrEqual(10, 7, 0) ||
-      SystemUtil::IsWindows8OrLater()) {
-    use_emoji_conversion_default = true;
-  }
-#endif  // __native_client__
-
-  if (use_emoji_conversion_default &&
+  if (GetPlatformSpecificDefaultEmojiSetting() &&
       !stored_config_.has_use_emoji_conversion()) {
     stored_config_.set_use_emoji_conversion(true);
   }
@@ -241,10 +233,15 @@ void ConfigHandlerImpl::SetConfigFileName(const string &filename) {
 }
 
 string ConfigHandlerImpl::GetConfigFileName() {
-  // Copies filename_ string using c_str() here to prevent Copy-On-Write issues
-  // in multi-thread environment.
+#ifdef __native_client__
+  // Copies filename_ string here to prevent Copy-On-Write issues in
+  // multi-thread environment.
   // See: http://stackoverflow.com/questions/1661154/c-stdstring-in-a-multi-threaded-program/
-  return filename_.c_str();
+  // TODO(hsumita): Remove this hack if not necessary.
+  return string(filename_.data(), filename_.size());
+#else
+  return filename_;
+#endif  // __native_client__
 }
 }  // namespace
 
@@ -252,7 +249,7 @@ const Config &ConfigHandler::GetConfig() {
   return GetConfigHandlerImpl()->GetConfig();
 }
 
-// return current Config
+// Returns current Config
 bool ConfigHandler::GetConfig(Config *config) {
   return GetConfigHandlerImpl()->GetConfig(config);
 }
@@ -261,28 +258,23 @@ const Config &ConfigHandler::GetStoredConfig() {
   return GetConfigHandlerImpl()->GetStoredConfig();
 }
 
-// return Stored Config
+// Returns Stored Config
 bool ConfigHandler::GetStoredConfig(Config *config) {
   return GetConfigHandlerImpl()->GetStoredConfig(config);
 }
 
-// set config
 bool ConfigHandler::SetConfig(const Config &config) {
   return GetConfigHandlerImpl()->SetConfig(config);
 }
 
-// set overriding config
+// Sets overriding config
 void ConfigHandler::SetImposedConfig(const Config &config) {
   GetConfigHandlerImpl()->SetImposedConfig(config);
 }
 
 void ConfigHandler::GetDefaultConfig(Config *config) {
   config->Clear();
-#ifdef OS_MACOSX
-  config->set_session_keymap(Config::KOTOERI);
-#else  // OS_MACOSX
-  config->set_session_keymap(Config::MSIME);
-#endif  // OS_MACOSX
+  config->set_session_keymap(ConfigHandler::GetDefaultKeyMap());
 
   const Config::CharacterForm kFullWidth = Config::FULL_WIDTH;
   const Config::CharacterForm kLastForm = Config::LAST_FORM;
@@ -305,15 +297,11 @@ void ConfigHandler::GetDefaultConfig(Config *config) {
   AddCharacterFormRule("<>=+-/*", kFullWidth, kLastForm, config);
   AddCharacterFormRule("?!", kFullWidth, kLastForm, config);
 
-#ifdef OS_ANDROID
-#ifdef CHANNEL_DEV
-  config->mutable_general_config()
-      ->set_upload_usage_stats(true);
-#endif  // CHANNEL_DEV
-#endif  // OS_ANDROID
+#if defined(OS_ANDROID) && defined(CHANNEL_DEV)
+  config->mutable_general_config()->set_upload_usage_stats(true);
+#endif  // OS_ANDROID && CHANNEL_DEV
 
-  if (SystemUtil::MacOSVersionIsGreaterOrEqual(10, 7, 0) ||
-      SystemUtil::IsWindows8OrLater()) {
+  if (GetPlatformSpecificDefaultEmojiSetting()) {
     config->set_use_emoji_conversion(true);
   }
 }
@@ -340,6 +328,15 @@ void ConfigHandler::SetMetaData(Config *config) {
   general_config->set_platform(SystemUtil::GetOSVersionString());
 }
 
-}  // namespace config
+Config::SessionKeymap ConfigHandler::GetDefaultKeyMap() {
+#if defined(OS_MACOSX)
+  return config::Config::KOTOERI;
+#elif defined(__native_client__)  // OS_MACOSX
+  return config::Config::CHROMEOS;
+#else  // OS_MACOSX or __native_client__
+  return config::Config::MSIME;
+#endif  // OS_MACOSX or __native_client__
+}
 
+}  // namespace config
 }  // namespace mozc
