@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -33,9 +33,10 @@
 #include <string>
 #include <vector>
 
-#include "base/base.h"
 #include "base/freelist.h"
 #include "base/number_util.h"
+#include "base/port.h"
+#include "base/scoped_ptr.h"
 #include "base/util.h"
 #include "converter/node.h"
 #include "converter/segments.h"
