@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -31,11 +31,11 @@
 
 #include <string>
 #include <vector>
-#include <sstream>
+#include <sstream>  // NOLINT
 
-#include "base/base.h"
 #include "base/file_stream.h"
 #include "base/logging.h"
+#include "base/port.h"
 #include "base/string_piece.h"
 #include "base/text_normalizer.h"
 #include "base/util.h"
