@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -35,24 +35,18 @@
 
 namespace mozc {
 
-class NodeAllocatorInterface;
-
 class UserDictionaryStub : public DictionaryInterface {
  public:
-  virtual bool HasValue(const StringPiece value) const {
+  virtual bool HasKey(StringPiece key) const {
+    return false;
+  }
+  virtual bool HasValue(StringPiece value) const {
     return false;
   }
 
-  virtual Node *LookupPredictiveWithLimit(
-      const char *str, int size, const Limit &limit,
-      NodeAllocatorInterface *allocator) const {
-    return NULL;
-  }
-
-  virtual Node *LookupPredictive(const char *str, int size,
-                                 NodeAllocatorInterface *allocator) const {
-    return NULL;
-  }
+  virtual void LookupPredictive(
+      StringPiece key, bool use_kana_modifier_insensitive_lookup,
+      Callback *callback) const {}
 
   virtual void LookupPrefix(
       StringPiece key, bool use_kana_modifier_insensitive_lookup,
@@ -60,10 +54,8 @@ class UserDictionaryStub : public DictionaryInterface {
 
   virtual void LookupExact(StringPiece key, Callback *callback) const {}
 
-  virtual Node *LookupReverse(const char *str, int size,
-                              NodeAllocatorInterface *allocator) const {
-    return NULL;
-  }
+  virtual void LookupReverse(StringPiece str, NodeAllocatorInterface *allocator,
+                             Callback *callback) const {}
 
   virtual bool LookupComment(StringPiece key, StringPiece value,
                              string *comment) const {
