@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -29,8 +29,8 @@
 
 #include "engine/engine.h"
 
-#include "base/base.h"
 #include "base/logging.h"
+#include "base/port.h"
 #include "converter/connector_base.h"
 #include "converter/converter.h"
 #include "converter/converter_interface.h"
@@ -84,6 +84,8 @@ class UserDataManagerImpl : public UserDataManagerInterface {
  private:
   PredictorInterface *predictor_;
   RewriterInterface *rewriter_;
+
+  DISALLOW_COPY_AND_ASSIGN(UserDataManagerImpl);
 };
 
 UserDataManagerImpl::~UserDataManagerImpl() {}
@@ -134,7 +136,8 @@ Engine::~Engine() {}
 void Engine::Init(
     const DataManagerInterface *data_manager,
     PredictorInterface *(*predictor_factory)(PredictorInterface *,
-                                             PredictorInterface *)) {
+                                             PredictorInterface *),
+    bool enable_content_word_learning) {
   CHECK(data_manager);
   CHECK(predictor_factory);
 
@@ -223,7 +226,8 @@ void Engine::Init(
     PredictorInterface *user_history_predictor =
         new UserHistoryPredictor(dictionary_.get(),
                                  data_manager->GetPOSMatcher(),
-                                 suppression_dictionary_.get());
+                                 suppression_dictionary_.get(),
+                                 enable_content_word_learning);
     CHECK(user_history_predictor);
 
     predictor_ = (*predictor_factory)(dictionary_predictor,
