@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -111,8 +111,6 @@ class MockPredictor : public PredictorInterface {
  public:
   MockPredictor() {}
   virtual ~MockPredictor() {}
-
-  MOCK_CONST_METHOD1(Predict, bool(Segments *segments));
   MOCK_CONST_METHOD2(
       PredictForRequest,
       bool(const ConversionRequest &request, Segments *segments));
@@ -295,7 +293,8 @@ TEST_F(PredictorTest, CallPredictorsForMobilePartialPrediction) {
           new UserHistoryPredictor(
               &dictionary_mock,
               UserPosManager::GetUserPosManager()->GetPOSMatcher(),
-              Singleton<SuppressionDictionary>::get())));
+              Singleton<SuppressionDictionary>::get(),
+              true)));
   Segments segments;
   {
     segments.set_request_type(Segments::PARTIAL_PREDICTION);
