@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,6 @@
 #include "config/config.pb.h"
 #include "config/config_handler.h"
 #include "converter/conversion_request.h"
-#include "converter/node_allocator.h"
 #include "converter/segments.h"
 #include "dictionary/dictionary_interface.h"
 #include "dictionary/pos_matcher.h"
@@ -133,6 +132,19 @@ bool IsPunctuation(const string &value) {
           value == "\xEF\xBC\x8C" || value == "\xEF\xBC\x8E");
 }
 
+bool IsSentenceLikeCandidate(const Segment::Candidate &candidate) {
+  // A sentence should have a long reading.  Length check is done using key to
+  // absorb length difference in value variation, e.g.,
+  // "〜ください" and "〜下さい".
+  if (candidate.value.empty() || Util::CharsLen(candidate.key) < 8) {
+    return false;
+  }
+  // Our primary target sentence ends with Hiragana, e.g., "〜ます".
+  const ConstChar32ReverseIterator iter(candidate.value);
+  bool ret = Util::GetScriptType(iter.Get()) == Util::HIRAGANA;
+  return ret;
+}
+
 // Return romanaized string.
 string ToRoman(const string &str) {
   string result;
@@ -380,11 +392,13 @@ class UserHistoryPredictorSyncer : public Thread {
 UserHistoryPredictor::UserHistoryPredictor(
     const DictionaryInterface *dictionary,
     const POSMatcher *pos_matcher,
-    const SuppressionDictionary *suppression_dictionary)
+    const SuppressionDictionary *suppression_dictionary,
+    bool enable_content_word_learning)
     : dictionary_(dictionary),
       pos_matcher_(pos_matcher),
       suppression_dictionary_(suppression_dictionary),
       predictor_name_("UserHistoryPredictor"),
+      content_word_learning_enabled_(enable_content_word_learning),
       updated_(false),
       dic_(new DicCache(UserHistoryPredictor::cache_size())) {
   AsyncLoad();  // non-blocking
@@ -911,10 +925,10 @@ bool UserHistoryPredictor::LookupEntry(
   const Entry *last_entry = NULL;
 
   // last_access_time of the left-closest content word.
-  uint32 left_last_access_time = 0;
+  uint64 left_last_access_time = 0;
 
   // last_access_time of the left-most content word.
-  uint32 left_most_last_access_time = 0;
+  uint64 left_most_last_access_time = 0;
 
   // Example: [a|B|c|D]
   // a,c: functional word
@@ -966,7 +980,7 @@ bool UserHistoryPredictor::LookupEntry(
     string key = entry->key();
     string value = entry->value();
     const Entry *current_entry = entry;
-    set<uint64> seen;
+    set<uint32> seen;
     seen.insert(EntryFingerprint(*current_entry));
     // Until target entry gets longer than input_key.
     while (key.size() <= input_key.size()) {
@@ -1439,7 +1453,7 @@ void UserHistoryPredictor::InsertNextEntry(
     target_next_entry = entry->add_next_entries();
   } else {
     // Otherwise, find the oldest next_entry.
-    uint32 last_access_time = UINT_MAX;
+    uint64 last_access_time = kuint64max;
     for (int i = 0; i < entry->next_entries_size(); ++i) {
       // already has the same id
       if (next_entry.entry_fp() == entry->next_entries(i).entry_fp()) {
@@ -1509,7 +1523,7 @@ void UserHistoryPredictor::InsertEvent(EntryType type) {
     return;
   }
 
-  const uint32 last_access_time = static_cast<uint32>(Util::GetTime());
+  const uint64 last_access_time = Util::GetTime();
   const uint32 dic_key = Fingerprint("", "", type);
 
   CHECK(dic_.get());
@@ -1531,7 +1545,7 @@ void UserHistoryPredictor::Insert(const string &key,
                                   const string &description,
                                   bool is_suggestion_selected,
                                   uint32 next_fp,
-                                  uint32 last_access_time,
+                                  uint64 last_access_time,
                                   Segments *segments) {
   if (key.empty() || value.empty() ||
       key.size() > kMaxStringLength ||
@@ -1619,7 +1633,7 @@ void UserHistoryPredictor::Finish(Segments *segments) {
   }
 
   const bool is_suggestion = segments->request_type() != Segments::CONVERSION;
-  const uint32 last_access_time = static_cast<uint32>(Util::GetTime());
+  const uint64 last_access_time = Util::GetTime();
 
   // If user inputs a punctuation just after some long sentence,
   // we make a new candidate by concatinating the top element in LRU and
@@ -1627,26 +1641,25 @@ void UserHistoryPredictor::Finish(Segments *segments) {
   // the long sentence user input before.
   // This is a fix for http://b/issue?id=2216838
   if (dic_->Head() != NULL &&
+      dic_->Head()->value.last_access_time() + 5 > last_access_time &&
+      // Check if the current value is a punctuation.
       segments->conversion_segments_size() == 1 &&
-      segments->history_segments_size() > 0 &&
       segments->conversion_segment(0).candidates_size() > 0 &&
+      IsPunctuation(segments->conversion_segment(0).candidate(0).value) &&
+      // Check if the previous value looks like a sentence.
+      segments->history_segments_size() > 0 &&
       segments->history_segment(
           segments->history_segments_size() - 1).candidates_size() > 0 &&
-      Util::CharsLen(
-          segments->conversion_segment(0).candidate(0).value) == 1 &&
-      IsPunctuation(
-          segments->conversion_segment(0).candidate(0).value) &&
-      dic_->Head()->value.last_access_time() + 5 > last_access_time) {
+      IsSentenceLikeCandidate(segments->history_segment(
+          segments->history_segments_size() - 1).candidate(0))) {
     const Entry *entry = &(dic_->Head()->value);
     DCHECK(entry);
     const string &last_value =
         segments->history_segment(
             segments->history_segments_size() - 1).candidate(0).value;
-    // Check that value in head element of LRU ends with the candidate value
-    // in history segments.
-    if (last_value.size() <= entry->value().size() &&
-        entry->value().substr(entry->value().size() - last_value.size(),
-                              last_value.size()) == last_value) {
+    // Check if the head value in LRU ends with the candidate value in history
+    // segments.
+    if (Util::EndsWith(entry->value(), last_value)) {
       const Segment::Candidate &candidate =
           segments->conversion_segment(0).candidate(0);
       const string key = entry->key() + candidate.key;
@@ -1699,6 +1712,8 @@ void UserHistoryPredictor::MakeLearningSegments(
     DCHECK_LE(1, segment.candidates_size());
     learning_segment.key = segment.candidate(0).key;
     learning_segment.value = segment.candidate(0).value;
+    learning_segment.content_key = segment.candidate(0).content_key;
+    learning_segment.content_value = segment.candidate(0).content_value;
     learning_segment.description = GetDescription(segment.candidate(0));
     learning_segments->push_back_history_segment(learning_segment);
   }
@@ -1709,36 +1724,32 @@ void UserHistoryPredictor::MakeLearningSegments(
       SegmentForLearning learning_segment;
       learning_segment.key = candidate.key;
       learning_segment.value = candidate.value;
+      learning_segment.content_key = candidate.content_key;
+      learning_segment.content_value = candidate.content_value;
       learning_segment.description = GetDescription(candidate);
       learning_segments->push_back_conversion_segment(learning_segment);
     } else {
-      int key_start_pos = 0, value_start_pos = 0;
-      for (size_t j = 0; j < candidate.inner_segment_boundary.size(); ++j) {
-        const int key_len = candidate.inner_segment_boundary[j].first;
-        const int value_len = candidate.inner_segment_boundary[j].second;
-        SegmentForLearning learning_segment;
-        Util::SubString(candidate.key, key_start_pos,
-                        key_len, &learning_segment.key);
-        Util::SubString(candidate.value, value_start_pos,
-                        value_len, &learning_segment.value);
+      SegmentForLearning learning_segment;
+      for (Segment::Candidate::InnerSegmentIterator iter(&candidate);
+           !iter.Done(); iter.Next()) {
+        iter.GetKey().CopyToString(&learning_segment.key);
+        iter.GetValue().CopyToString(&learning_segment.value);
+        iter.GetContentKey().CopyToString(&learning_segment.content_key);
+        iter.GetContentValue().CopyToString(&learning_segment.content_value);
         learning_segments->push_back_conversion_segment(learning_segment);
-        key_start_pos += key_len;
-        value_start_pos += value_len;
       }
-      DCHECK_EQ(key_start_pos, Util::CharsLen(candidate.key));
-      DCHECK_EQ(value_start_pos, Util::CharsLen(candidate.value));
     }
   }
 }
 
 void UserHistoryPredictor::InsertHistory(bool is_suggestion_selected,
-                                         uint32 last_access_time,
+                                         uint64 last_access_time,
                                          Segments *segments) {
   SegmentsForLearning learning_segments;
   MakeLearningSegments(*segments, &learning_segments);
 
   string all_key, all_value;
-  set<uint64> seen;
+  set<uint32> seen;
   bool this_was_seen = false;
   const size_t history_segments_size =
       learning_segments.history_segments_size();
@@ -1774,9 +1785,18 @@ void UserHistoryPredictor::InsertHistory(bool is_suggestion_selected,
            segment.description,
            is_suggestion_selected, next_fp_to_set,
            last_access_time, segments);
+    if (content_word_learning_enabled_ &&
+        segment.content_key != segment.key &&
+        segment.content_value != segment.value) {
+      Insert(segment.content_key,
+             segment.content_value,
+             segment.description,
+             is_suggestion_selected, 0,
+             last_access_time, segments);
+    }
   }
 
-  // insert all_key/all_value
+  // Insert all_key/all_value
   if (learning_segments.conversion_segments_size() > 1 &&
       !all_key.empty() && !all_value.empty()) {
     Insert(all_key, all_value, "",
@@ -1784,24 +1804,30 @@ void UserHistoryPredictor::InsertHistory(bool is_suggestion_selected,
            0, last_access_time, segments);
   }
 
-  // make a link from the right most history_segment to
-  // the left most segment or entire user input.
+  // Make a link from the last history_segment to the first conversion segment
+  // or to the entire user input.
   if (learning_segments.history_segments_size() > 0 &&
       learning_segments.conversion_segments_size() > 0) {
-    Entry *history_entry =
-        dic_->MutableLookupWithoutInsert(
-            LearningSegmentFingerprint(
-                learning_segments.history_segment(
-                    segments->history_segments_size() - 1)));
-
+    const SegmentForLearning &history_segment =
+        learning_segments.history_segment(
+            segments->history_segments_size() - 1);
+    const SegmentForLearning &conversion_segment =
+        learning_segments.conversion_segment(0);
+    // Don't learn a link from/to a punctuation.  Note that another piece of
+    // code handles learning for (sentence + punctuation) form; see Finish().
+    if (IsPunctuation(history_segment.value) ||
+        IsPunctuation(conversion_segment.value)) {
+      return;
+    }
+    Entry *history_entry = dic_->MutableLookupWithoutInsert(
+        LearningSegmentFingerprint(history_segment));
     NextEntry next_entry;
     if (segments->request_type() == Segments::CONVERSION) {
-      next_entry.set_entry_fp(
-          LearningSegmentFingerprint(learning_segments.conversion_segment(0)));
+      next_entry.set_entry_fp(LearningSegmentFingerprint(conversion_segment));
       InsertNextEntry(next_entry, history_entry);
     }
 
-    // entire user input or SUGGESTION
+    // Entire user input or SUGGESTION
     if (segments->request_type() != Segments::CONVERSION ||
         learning_segments.conversion_segments_size() > 1) {
       next_entry.set_entry_fp(Fingerprint(all_key, all_value));
@@ -1829,7 +1855,6 @@ void UserHistoryPredictor::Revert(Segments *segments) {
   }
 }
 
-// type
 // static
 UserHistoryPredictor::MatchType UserHistoryPredictor::GetMatchType(
     const string &lstr, const string &rstr) {
