@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -144,7 +144,7 @@ class TableLayout : public TableLayoutInterface {
 
   // Fix the layout and calculate the total size.
   void FreezeLayout();
-  int IsLayoutFrozen() const;
+  bool IsLayoutFrozen() const;
 
   // Get the rect which is bounding the specified cell.
   // This rect does not include RowRectPadding.
