@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -39,16 +39,9 @@
 #include "base/port.h"
 #include "base/singleton.h"
 #include "config/config.pb.h"
+#include "session/commands.pb.h"
 
 namespace mozc {
-namespace commands {
-class KeyEvent;
-}  // namespace commands
-
-namespace config {
-class Config;
-}  // namespace config
-
 namespace session {
 
 class KeyEventTransformer {
