@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -57,16 +57,14 @@ static const char kATOKKeyMapFile[] = "system://atok.tsv";
 static const char kKotoeriKeyMapFile[] = "system://kotoeri.tsv";
 static const char kCustomKeyMapFile[] = "user://keymap.tsv";
 static const char kMobileKeyMapFile[] = "system://mobile.tsv";
+static const char kChromeOsKeyMapFile[] = "system://chromeos.tsv";
+}  // namespace
 
-// TODO(team): Investigate if "InputModeX" commands should be
-//     functional on Android or not. Remove the condition if available.
-#if defined(OS_MACOSX) || defined(OS_ANDROID)
-const bool kInputModeXCommandSupported = false;
+#if defined(OS_MACOSX)
+const bool KeyMapManager::kInputModeXCommandSupported = false;
 #else
-const bool kInputModeXCommandSupported = true;
-#endif
-
-}  // anonymous namespace
+const bool KeyMapManager::kInputModeXCommandSupported = true;
+#endif  // OS_MACOSX
 
 KeyMapManager::KeyMapManager()
     : keymap_(config::Config::NONE) {
@@ -100,7 +98,8 @@ bool KeyMapManager::ReloadWithKeymap(
     const string &custom_keymap_table = GET_CONFIG(custom_keymap_table);
     if (custom_keymap_table.empty()) {
       LOG(WARNING) << "custom_keymap_table is empty. use default setting";
-      const char *default_keymapfile = GetKeyMapFileName(GetDefaultKeyMap());
+      const char *default_keymapfile = GetKeyMapFileName(
+          config::ConfigHandler::GetDefaultKeyMap());
       return LoadFile(default_keymapfile);
     }
 #ifndef NO_LOGGING
@@ -121,7 +120,8 @@ bool KeyMapManager::ReloadWithKeymap(
     return true;
   }
 
-  const char *default_keymapfile = GetKeyMapFileName(GetDefaultKeyMap());
+  const char *default_keymapfile = GetKeyMapFileName(
+      config::ConfigHandler::GetDefaultKeyMap());
   return LoadFile(default_keymapfile);
 }
 
@@ -137,6 +137,8 @@ const char *KeyMapManager::GetKeyMapFileName(
       return kMSIMEKeyMapFile;
     case config::Config::KOTOERI:
       return kKotoeriKeyMapFile;
+    case config::Config::CHROMEOS:
+      return kChromeOsKeyMapFile;
     case config::Config::CUSTOM:
       return kCustomKeyMapFile;
     case config::Config::NONE:
@@ -144,26 +146,19 @@ const char *KeyMapManager::GetKeyMapFileName(
       // should not appear here.
       LOG(ERROR) << "Keymap type: " << keymap
                  << " appeared at key map initialization.";
-      const config::Config::SessionKeymap default_keymap = GetDefaultKeyMap();
+      const config::Config::SessionKeymap default_keymap =
+          config::ConfigHandler::GetDefaultKeyMap();
       DCHECK(default_keymap == config::Config::ATOK ||
              default_keymap == config::Config::MOBILE ||
              default_keymap == config::Config::MSIME ||
              default_keymap == config::Config::KOTOERI ||
+             default_keymap == config::Config::CHROMEOS ||
              default_keymap == config::Config::CUSTOM);
       // should never make loop.
       return GetKeyMapFileName(default_keymap);
   }
 }
 
-// static
-config::Config::SessionKeymap KeyMapManager::GetDefaultKeyMap() {
-#ifdef OS_MACOSX
-  return config::Config::KOTOERI;
-#else  // OS_MACOSX
-  return config::Config::MSIME;
-#endif  // OS_MACOSX
-}
-
 bool KeyMapManager::LoadFile(const char *filename) {
   scoped_ptr<istream> ifs(ConfigFileStream::LegacyOpen(filename));
   if (ifs.get() == NULL) {
@@ -219,22 +214,14 @@ bool KeyMapManager::AddCommand(const string &state_name,
                                const string &key_event_name,
                                const string &command_name) {
 #ifdef NO_LOGGING  // means RELEASE BUILD
-  // On the release build, we do not support the Abort and ReportBug
+  // On the release build, we do not support the ReportBug
   // commands.  Note, true is returned as the arguments are
   // interpreted properly.
-  if (command_name == "Abort" || command_name == "ReportBug") {
+  if (command_name == "ReportBug") {
     return true;
   }
 #endif  // NO_LOGGING
 
-#ifndef DEBUG
-  // Only debug build supports the Abort command.  Note, true is
-  // returned as the arguments are interpreted properly.
-  if (command_name == "Abort") {
-    return true;
-  }
-#endif  // DEBUG
-
   commands::KeyEvent key_event;
   if (!KeyParser::ParseKey(key_event_name, &key_event)) {
     return false;
@@ -378,32 +365,29 @@ void KeyMapManager::RegisterConversionCommand(
 
 void KeyMapManager::InitCommandData() {
   RegisterDirectCommand("IMEOn", DirectInputState::IME_ON);
-  // Support InputMode command only on Windows for now.
-  // TODO(toshiyuki): delete #ifdef when we support them on Mac, and
-  // activate SessionTest.InputModeConsumedForTestSendKey.
-#ifdef OS_WIN
-  RegisterDirectCommand("InputModeHiragana",
-                        DirectInputState::INPUT_MODE_HIRAGANA);
-  RegisterDirectCommand("InputModeFullKatakana",
-                        DirectInputState::INPUT_MODE_FULL_KATAKANA);
-  RegisterDirectCommand("InputModeHalfKatakana",
-                        DirectInputState::INPUT_MODE_HALF_KATAKANA);
-  RegisterDirectCommand("InputModeFullAlphanumeric",
-                        DirectInputState::INPUT_MODE_FULL_ALPHANUMERIC);
-  RegisterDirectCommand("InputModeHalfAlphanumeric",
-                        DirectInputState::INPUT_MODE_HALF_ALPHANUMERIC);
-#else
-  RegisterDirectCommand("InputModeHiragana",
-                        DirectInputState::NONE);
-  RegisterDirectCommand("InputModeFullKatakana",
-                        DirectInputState::NONE);
-  RegisterDirectCommand("InputModeHalfKatakana",
-                        DirectInputState::NONE);
-  RegisterDirectCommand("InputModeFullAlphanumeric",
-                        DirectInputState::NONE);
-  RegisterDirectCommand("InputModeHalfAlphanumeric",
-                        DirectInputState::NONE);
-#endif  // OS_WIN
+  if (kInputModeXCommandSupported) {
+    RegisterDirectCommand("InputModeHiragana",
+                          DirectInputState::INPUT_MODE_HIRAGANA);
+    RegisterDirectCommand("InputModeFullKatakana",
+                          DirectInputState::INPUT_MODE_FULL_KATAKANA);
+    RegisterDirectCommand("InputModeHalfKatakana",
+                          DirectInputState::INPUT_MODE_HALF_KATAKANA);
+    RegisterDirectCommand("InputModeFullAlphanumeric",
+                          DirectInputState::INPUT_MODE_FULL_ALPHANUMERIC);
+    RegisterDirectCommand("InputModeHalfAlphanumeric",
+                          DirectInputState::INPUT_MODE_HALF_ALPHANUMERIC);
+  } else {
+    RegisterDirectCommand("InputModeHiragana",
+                          DirectInputState::NONE);
+    RegisterDirectCommand("InputModeFullKatakana",
+                          DirectInputState::NONE);
+    RegisterDirectCommand("InputModeHalfKatakana",
+                          DirectInputState::NONE);
+    RegisterDirectCommand("InputModeFullAlphanumeric",
+                          DirectInputState::NONE);
+    RegisterDirectCommand("InputModeHalfAlphanumeric",
+                          DirectInputState::NONE);
+  }
   RegisterDirectCommand("Reconvert",
                         DirectInputState::RECONVERT);
 
@@ -475,10 +459,6 @@ void KeyMapManager::InitCommandData() {
   RegisterPrecompositionCommand("PredictAndConvert",
                                 PrecompositionState::PREDICT_AND_CONVERT);
 
-#ifdef DEBUG  // only for debugging
-  RegisterPrecompositionCommand("Abort", PrecompositionState::ABORT);
-#endif  // DEBUG
-
   // Composition
   RegisterCompositionCommand("IMEOff", CompositionState::IME_OFF);
   RegisterCompositionCommand("IMEOn", CompositionState::IME_ON);
@@ -565,9 +545,6 @@ void KeyMapManager::InitCommandData() {
     RegisterCompositionCommand("InputModeHalfAlphanumeric",
                                CompositionState::NONE);
   }
-#ifdef DEBUG  // only for debugging
-  RegisterCompositionCommand("Abort", CompositionState::ABORT);
-#endif  // DEBUG
 
   // Conversion
   RegisterConversionCommand("IMEOff", ConversionState::IME_OFF);
@@ -665,9 +642,6 @@ void KeyMapManager::InitCommandData() {
 #ifndef NO_LOGGING  // means NOT RELEASE build
   RegisterConversionCommand("ReportBug", ConversionState::REPORT_BUG);
 #endif  // NO_LOGGING
-#ifdef DEBUG  // only for dubugging
-  RegisterConversionCommand("Abort", ConversionState::ABORT);
-#endif  // DEBUG
 }
 
 bool KeyMapManager::GetCommandDirect(
