@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -128,9 +128,13 @@ class SessionHandlerScenarioTest : public SessionHandlerTestBase,
 const char *kScenarioFileList[] = {
 #define DATA_DIR "data/test/session/scenario/"
   DATA_DIR "auto_partial_suggestion.txt",
+  DATA_DIR "b12751061_scenario.txt",
+  DATA_DIR "b16123009_scenario.txt",
+  DATA_DIR "b18112966_scenario.txt",
   DATA_DIR "b7132535_scenario.txt",
   DATA_DIR "b7321313_scenario.txt",
   DATA_DIR "b7548679_scenario.txt",
+  DATA_DIR "b8690065_scenario.txt",
   DATA_DIR "b8703702_scenario.txt",
   DATA_DIR "change_request.txt",
   DATA_DIR "clear_user_prediction.txt",
@@ -147,12 +151,11 @@ const char *kScenarioFileList[] = {
   DATA_DIR "convert_from_hiragana_to_t13n.txt",
   DATA_DIR "delete_history.txt",
   DATA_DIR "desktop_t13n_candidates.txt",
-#if !defined(OS_MACOSX) && !defined(OS_ANDROID)
-  // "InputModeX" commands are not supported on Mac and Android.
+#if !defined(OS_MACOSX)
+  // "InputModeX" commands are not supported on Mac.
   // Mac: We do not have the way to change the mode indicator from IME.
-  // Android: Input mode change is performed by changing software keyboard.
   DATA_DIR "input_mode.txt",
-#endif
+#endif  // !OS_MACOSX
   DATA_DIR "insert_characters.txt",
   DATA_DIR "mobile_qwerty_transliteration_scenario.txt",
   DATA_DIR "mobile_t13n_candidates.txt",
@@ -165,6 +168,7 @@ const char *kScenarioFileList[] = {
   DATA_DIR "segment_focus.txt",
   DATA_DIR "segment_width.txt",
   DATA_DIR "twelvekeys_switch_inputmode_scenario.txt",
+  DATA_DIR "twelvekeys_toggle_flick_alphabet_scenario.txt",
   DATA_DIR "twelvekeys_toggle_hiragana_preedit_scenario.txt",
   DATA_DIR "undo.txt",
 #undef DATA_DIR
