@@ -1,4 +1,4 @@
-// Copyright 2010-2014, Google Inc.
+// Copyright 2010-2015, Google Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -75,12 +75,8 @@ DEFINE_bool(use_mozc_renderer, true,
 
 namespace {
 
-// A key which associates an IBusProperty object with MozcEngineProperty.
-const char kGObjectDataKey[] = "ibus-mozc-aux-data";
-// An ID for a candidate which is not associated with a text.
+// The ID for candidates which are not associated with texts.
 const int32 kBadCandidateId = -1;
-// The ibus-memconf section name in which we're interested.
-const char kMozcSectionName[] = "engine/Mozc";
 
 #ifdef ENABLE_GTK_RENDERER
 const char kMozcPanelSectionName[] = "panel";
@@ -223,24 +219,33 @@ bool GetSurroundingText(IBusEngine *engine,
     return false;
   }
 
-  const uint32 selection_start = min(cursor_pos, anchor_pos);
-  const uint32 selection_length = abs(info->relative_selected_length);
-  info->preceding_text = surrounding_text.substr(0, selection_start);
-  Util::SubString(surrounding_text,
-                  selection_start,
-                  selection_length,
-                  &info->selection_text);
-  info->following_text = surrounding_text.substr(
-      selection_start + selection_length);
+  const size_t selection_start = min(cursor_pos, anchor_pos);
+  const size_t selection_length = abs(info->relative_selected_length);
+  Util::SubStringPiece(surrounding_text, 0, selection_start)
+      .CopyToString(&info->preceding_text);
+  Util::SubStringPiece(surrounding_text, selection_start, selection_length)
+      .CopyToString(&info->selection_text);
+  Util::SubStringPiece(surrounding_text, selection_start + selection_length)
+      .CopyToString(&info->following_text);
   return true;
 }
 
+std::unique_ptr<client::ClientInterface> CreateAndConfigureClient() {
+  std::unique_ptr<client::ClientInterface> client(
+      client::ClientFactory::NewClient());
+  // Currently client capability is fixed.
+  commands::Capability capability;
+  capability.set_text_deletion(commands::Capability::DELETE_PRECEDING_TEXT);
+  client->set_client_capability(capability);
+  return client;
+}
+
 }  // namespace
 
 MozcEngine::MozcEngine()
     : last_sync_time_(Util::GetTime()),
       key_event_handler_(new KeyEventHandler),
-      client_(client::ClientFactory::NewClient()),
+      client_(CreateAndConfigureClient()),
 #ifdef MOZC_ENABLE_X11_SELECTION_MONITOR
       selection_monitor_(SelectionMonitorFactory::Create(1024)),
 #endif  // MOZC_ENABLE_X11_SELECTION_MONITOR
@@ -250,16 +255,9 @@ MozcEngine::MozcEngine()
 #ifdef ENABLE_GTK_RENDERER
       gtk_candidate_window_handler_(new GtkCandidateWindowHandler(
           new renderer::RendererClient())),
-#else
-      gtk_candidate_window_handler_(NULL),
 #endif  // ENABLE_GTK_RENDERER
       ibus_candidate_window_handler_(new IBusCandidateWindowHandler()),
       preedit_method_(config::Config::ROMAN) {
-  // Currently client capability is fixed.
-  commands::Capability capability;
-  capability.set_text_deletion(commands::Capability::DELETE_PRECEDING_TEXT);
-  client_->set_client_capability(capability);
-
 #ifdef MOZC_ENABLE_X11_SELECTION_MONITOR
   if (selection_monitor_.get() != NULL) {
     selection_monitor_->StartMonitoring();
@@ -333,9 +331,14 @@ void MozcEngine::FocusOut(IBusEngine *engine) {
   GetCandidateWindowHandler(engine)->Hide(engine);
   property_handler_->ResetContentType(engine);
 
-  // Do not call SubmitSession or RevertSession. Preedit string will commit on
-  // Focus Out event automatically by ibus_engine_update_preedit_text_with_mode
-  // which called in UpdatePreedit method.
+  // Note that the preedit string (if any) will be committed by IBus runtime
+  // because we are specifying |IBUS_ENGINE_PREEDIT_COMMIT| flag to
+  // |ibus_engine_update_preedit_text_with_mode|. All we need to do here is
+  // simply resetting the current session in case there is a non-empty
+  // preedit text. Note that |RevertSession| is supposed to do nothing when
+  // there is no preedit text.
+  // See https://code.google.com/p/mozc/issues/detail?id=255 for details.
+  RevertSession(engine);
   SyncData(false);
 }
 
@@ -666,6 +669,8 @@ bool MozcEngine::LaunchTool(const commands::Output &output) const {
 }
 
 void MozcEngine::RevertSession(IBusEngine *engine) {
+  // TODO(team): We should skip following actions when there is no on-going
+  // omposition.
   commands::SessionCommand command;
   command.set_type(commands::SessionCommand::REVERT);
   commands::Output output;
